@@ -1,5 +1,5 @@
 /*
-   Copyright 2005,2006,2007 Red Hat, Inc.
+   Copyright 2005,2006,2007,2011 Red Hat, Inc.
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -61,9 +61,11 @@ static mode_t override_umask;
  * identical as possible a copy in the destination tree. */
 static int
 copy_single_item(const char *source, const struct stat *sb,
-		 int flag, struct FTW *unused_s)
+		 int flag, struct FTW *status)
 {
-	int sfd, dfd, i;
+	uid_t uid = pwd->pw_uid;
+	gid_t gid = pwd->pw_gid;
+	int sfd, dfd, i, res;
 	char target[PATH_MAX + 1], newpath[PATH_MAX + 1];
 	unsigned char buf[BUFSIZ];
 	/* Generate the name of the new item. */
@@ -152,12 +154,24 @@ copy_single_item(const char *source, const struct stat *sb,
 		}
 		return 0;
 	case FTW_D:
+		/* It's the home directory itself. Don't give it to the
+		 * target user just yet to avoid potential race conditions
+		 * involving symlink attacks when we copy over the skeleton
+		 * tree. */
+		if (status->level == 0) {
+			uid = 0;
+			gid = 0;
+		}
+
 		/* It's a directory.  Make one with the same name and
 		 * permissions, but owned by the target user. */
-		if ((oddjob_selinux_mkdir(newpath,
-					  sb->st_mode & ~override_umask,
-					  pwd->pw_uid, pwd->pw_gid) != 0) &&
-		    (errno != EEXIST)) {
+		res = oddjob_selinux_mkdir(newpath,
+					   sb->st_mode & ~override_umask,
+					   uid, gid);
+
+		/* on unexpected errors, or if the home directory itself
+		 * suddenly already exists, abort the copy operation. */
+		if (res != 0 && (errno != EEXIST || status->level == 0)) {
 			return HANDLER_FAILURE;
 		}
 		return 0;
@@ -220,8 +234,14 @@ mkhomedir(const char *user, int flags)
 			}
 			/* Walk the template tree and make a copy. */
 			if (flags & FLAG_POPULATE) {
-				return nftw(get_skel_dir(), copy_single_item, 5,
-					    FTW_PHYS);
+				int res = nftw(get_skel_dir(), copy_single_item, 5,
+					       FTW_PHYS);
+				/* only now give ownership to the target user */
+				if (res == 0) {
+					res = chown(pwd->pw_dir, pwd->pw_uid, pwd->pw_gid);
+				}
+
+				return res;
 			} else {
 				if (stat(skel, &st) != 0) {
 					st.st_mode = S_IRWXU;
@@ -243,14 +263,52 @@ mkhomedir(const char *user, int flags)
 	return 0;
 }
 
+static mode_t
+get_umask(int *configured)
+{
+	FILE *fp;
+	char buf[BUFSIZ], *p, *end;
+	mode_t mask = umask(0777);
+	long tmp;
+
+	fp = fopen("/etc/login.defs", "r");
+	if (fp != NULL) {
+		while (fgets(buf, sizeof(buf), fp) != NULL) {
+			if (buf[0] == '#') {
+				continue;
+			}
+			buf[strcspn(buf, "\r\n")] = '\0';
+			p = buf + strspn(buf, " \t");
+			if (strncmp(p, "UMASK", 5) != 0) {
+				continue;
+			}
+			p += 5;
+			if (strspn(p, " \t") == 0) {
+				continue;
+			}
+			p += strspn(p, " \t");
+			tmp = strtol(p, &end, 0);
+			if ((end != NULL) && (*end == '\0')) {
+				mask = tmp;
+				if (configured) {
+					*configured = 1;
+				}
+				break;
+			}
+		}
+		fclose(fp);
+	}
+	return mask;
+}
+
 int
 main(int argc, char **argv)
 {
 	char **args, *p;
-	int i, flags = FLAG_POPULATE;
+	int i, configured_umask = 0, flags = FLAG_POPULATE;
 
 	openlog(PACKAGE "-mkhomedir", LOG_PID, LOG_DAEMON);
-	override_umask = umask(0777);
+	override_umask = get_umask(&configured_umask);
 	umask(override_umask);
 	skel_dir = "/etc/skel";
 
@@ -272,6 +330,7 @@ main(int argc, char **argv)
 					optarg);
 				return 1;
 			}
+			configured_umask = 0;
 			break;
 		default:
 			fprintf(stderr, "Valid options:\n"
@@ -281,8 +340,11 @@ main(int argc, char **argv)
 				"a directory.\n"
 				"-s PATH\tOverride the skeleton directory "
 				"path (\"%s\").\n"
-				"-u MASK\tOverride the default umask (0%03o).\n",
-				skel_dir, override_umask);
+				"-u MASK\tOverride the default umask (0%03o%s).\n",
+				skel_dir, override_umask,
+				configured_umask ?
+				", from /etc/login.defs" :
+				"");
 			return 1;
 		}
 	}
